/*
 * macutil.cpp
 *
 * various Unix compatibility calls for the Mac.
 *
 */

#include <OSUtils.h> /* for SetDateTime */
#include <CursorCtl.h>
#include <Time.h>
#include <time.h>
#include <ctype.h>
#include <String.h>
#include <Script.h>
#include <FixMath.h>
#include <Memory.h>
#include <Resources.h>

static long gmt_offset = 0;             /* Seconds offset */
static got_offset = 0;

get_gmt_offset()
{
  MachineLocation ml;
  time_t t;
  struct tm *tmp;

  ReadLocation(&ml);
  if (ml.latitude != 0 || ml.longitude != 0) {
    gmt_offset = ml.u.gmtDelta & 0x00ffffff; 
    if ((gmt_offset >> 23) == 1)
      gmt_offset |= 0xff000000;
  }
  else {
    /* Default to EST/EDT, depending */
    time (&t);
    tmp = localtime(&t);
    if (tmp->tm_mon < 3 || tmp->tm_mon > 9) 	/* EST */
      gmt_offset = -5 * 60 * 60;
    else if (tmp->tm_mon > 3 && tmp->tm_mon < 9) /* EDT */
      gmt_offset = -4 * 60 * 60;
    else if (tmp->tm_mon == 3) {		/* April */
      if (tmp->tm_mday > 7)
	gmt_offset = -4 * 60 * 60;		/* Apr 8 & above EDT */
      else if (tmp->tm_wday == 0) {		/* Sunday */
	if (tmp->tm_hour > 2)			/* After 2am is EDT */
	  gmt_offset = -4 * 60 * 60;
	else
	  gmt_offset = -5 * 60 * 60;
      } else if (tmp->tm_mday - tmp->tm_wday > 0) /* After Sunday */
	gmt_offset = -4 * 60 * 60;
      else
	gmt_offset = -5 * 60 * 60;
    } else {					/* October */
      if (tmp->tm_mday < 25)
	gmt_offset = -4 * 60 * 60;	/* First part is EDT */
      else if (tmp->tm_wday == 0) {		/* Sunday */
	if (tmp->tm_hour > 2)		/* After 2am is EST */
	  gmt_offset = -5 * 60 * 60;
	else
	  gmt_offset = -4 * 60 * 60;
      } else if (tmp->tm_mday - tmp->tm_wday > 24) /* After Sunday */
	gmt_offset = -5 * 60 * 60;
      else
	gmt_offset = -4 * 60 * 60;
    }
  }
  got_offset = 1;
}

#define MAC_OFFSET (66 * 365 * 24 * 60 * 60 + 17 * 60 * 60 * 24)

void mac_time_to_unix_time (time_t *time)
{
  if (!got_offset) get_gmt_offset();
  *time  -= (MAC_OFFSET + gmt_offset);
}

int strncasecmp (const char *a, const char *b, int n)
{
  const char *p = a;
  const char *q = b;
  
  for (p = a, q = b; ; p++, q++) {
    int diff;
    if (p == a + n) return 0;
    if (!(*p && *q)) return *p - *q;
    diff = tolower(*p) - tolower(*q);
    if (diff) return diff;
  }
}

